@@ -1,10 +1,10 @@
-from ploomber.dag import DAG
+from ploomber.dag.DAG import DAG
 from ploomber.env.env import Env
 from ploomber.env.decorators import load_env, with_env
 from ploomber.templates.SourceLoader import SourceLoader
 
 
-__version__ = '0.3.1'
+__version__ = '0.3.3'
 
 # Set default logging handler to avoid "No handler found" warnings.
 import logging
