@@ -16,6 +16,10 @@ from ploomber.ExceptionCollector import ExceptionCollector, ExceptionResult
 
 import traceback
 
+# TODO: support for show_progress, we can use a progress bar but we have
+# to modify the label since at any point more than one task might be
+# executing
+
 
 class TaskBuildWrapper:
     """
@@ -31,7 +35,7 @@ class TaskBuildWrapper:
     def __call__(self, *args, **kwargs):
         try:
             return self.task.build(**kwargs)
-        except Exception as e:
+        except Exception:
             return ExceptionResult(task_str=repr(self.task),
                                    traceback_str=traceback.format_exc())
 
@@ -56,7 +60,7 @@ class Parallel(Executor):
         self._logger = logging.getLogger(__name__)
         self._i = 0
 
-    def __call__(self, dag, **kwargs):
+    def __call__(self, dag, show_progress, task_kwargs):
         super().__call__(dag)
 
         if self.logging_directory:
@@ -78,7 +82,8 @@ class Parallel(Executor):
         # this is a bit confusing, so maybe change WaitingExecution
         # to WaitingBuild?
         for name in dag:
-            if dag[name].exec_status == TaskStatus.Executed:
+            if dag[name].exec_status in {TaskStatus.Executed,
+                                         TaskStatus.Skipped}:
                 done.append(dag[name])
 
         def callback(future):
@@ -89,7 +94,7 @@ class Parallel(Executor):
                                task.name)
             try:
                 result = future.result()
-            except BrokenProcessPool as e:
+            except BrokenProcessPool:
                 # ignore the error here but flag the task,
                 # so next_task is able to stop the iteration,
                 # when we call result after breaking the loop,
@@ -99,7 +104,7 @@ class Parallel(Executor):
                 if isinstance(result, ExceptionResult):
                     task.exec_status = TaskStatus.Errored
                 else:
-                    task.exec_status = result
+                    task.exec_status = TaskStatus.Executed
 
             done.append(task)
 
@@ -111,7 +116,7 @@ class Parallel(Executor):
             the DAG is done
             """
             for task in dag.values():
-                if task.exec_status == TaskStatus.Aborted:
+                if task.exec_status in {TaskStatus.Aborted}:
                     done.append(task)
                 elif task.exec_status == TaskStatus.BrokenProcessPool:
                     raise StopIteration
@@ -153,28 +158,26 @@ class Parallel(Executor):
                     break
                 else:
                     if task is not None:
-                        future = pool.submit(TaskBuildWrapper(task), **kwargs)
+                        future = pool.submit(TaskBuildWrapper(task), **task_kwargs)
                         future.add_done_callback(callback)
                         started.append(task)
                         future_mapping[future] = task
                         logging.info('Added %s to the pool...', task.name)
                         # time.sleep(3)
 
-        exps = ExceptionCollector([get_future_result(f, future_mapping)
-                                   for f, t
-                                   in future_mapping.items()
-                                   if isinstance(get_future_result(f,
-                                                 future_mapping),
-                                                 ExceptionResult)])
+        results = [get_future_result(f, future_mapping)
+                   for f in future_mapping.keys()]
+
+        exps = [r for r in results if isinstance(r, ExceptionResult)]
 
         if exps:
             raise DAGBuildError('DAG build failed, the following '
                                 'tasks crashed '
                                 '(corresponding downstream tasks aborted '
                                 'execution):\n{}'
-                                .format(str(exps)))
+                                .format(str(ExceptionCollector(exps))))
 
-    # __getstate__ and __setstate__ are needed to make this picklable
+        return results
 
     def __getstate__(self):
         state = self.__dict__.copy()
@@ -192,4 +195,5 @@ def get_future_result(future, future_mapping):
     try:
         return future.result()
     except BrokenProcessPool as e:
-        raise BrokenProcessPool('Broken pool {}'.format(future_mapping[future])) from e
+        raise BrokenProcessPool('Broken pool {}'.format(
+            future_mapping[future])) from e
