@@ -1,31 +1,27 @@
 """
-DAG executors
+Serial DAG executor: builds one task at a time
 """
-from datetime import datetime
 from multiprocessing import Pool
 import traceback
 import logging
 
 from tqdm.auto import tqdm
-from ploomber.Table import BuildReport, Row
 from ploomber.executors.Executor import Executor
 from ploomber.executors.LoggerHandler import LoggerHandler
-from ploomber.exceptions import DAGBuildError, TaskBuildError
+from ploomber.exceptions import DAGBuildError
 from ploomber.ExceptionCollector import ExceptionCollector
 from ploomber.constants import TaskStatus
 
 
 # TODO: add a SerialIterator executor
-# TODO: document how executors should handle errors, they need to catch
-# failures in task.build but also in t.exec_status = TaskStatus.Executed
-# to report on_finish errors
 
 
 class Serial(Executor):
     """Runs a DAG serially
+
     """
     # TODO: maybe add a parameter: stop on first exception, same for Parallel
-    # TODO: add option to run all tasks in a subprocess
+
     def __init__(self, logging_directory=None, logging_level=logging.INFO,
                  execute_callables_in_subprocess=True):
         self.logging_directory = logging_directory
@@ -33,7 +29,7 @@ class Serial(Executor):
         self._logger = logging.getLogger(__name__)
         self._execute_callables_in_subprocess = execute_callables_in_subprocess
 
-    def __call__(self, dag, **kwargs):
+    def __call__(self, dag, show_progress, task_kwargs):
         super().__call__(dag)
 
         if self.logging_directory:
@@ -45,47 +41,39 @@ class Serial(Executor):
         exceptions = ExceptionCollector()
         task_reports = []
 
-        pbar = tqdm(dag._topologically_sorted_iter(skip_aborted=True),
-                    total=len(dag))
+        if show_progress:
+            tasks = tqdm(dag.values(), total=len(dag))
+        else:
+            tasks = dag.values()
 
-        for t in pbar:
-            pbar.set_description('Building task "{}"'.format(t.name))
+        for t in tasks:
+            if t.exec_status in {TaskStatus.Skipped, TaskStatus.Aborted}:
+                continue
 
-            then = datetime.now()
+            if show_progress:
+                tasks.set_description('Building task "{}"'.format(t.name))
 
             try:
                 if (callable(t.source.value)
                         and self._execute_callables_in_subprocess):
-                    new_status = execute_in_subprocess(t, kwargs)
+                    report = execute_in_subprocess(t, task_kwargs)
                 else:
-                    new_status = t.build(**kwargs)
-            except Exception as e:
+                    report = t.build(**task_kwargs)
+            except Exception:
                 t.exec_status = TaskStatus.Errored
                 new_status = TaskStatus.Errored
                 tr = traceback.format_exc()
                 exceptions.append(traceback_str=tr, task_str=repr(t))
-
-                # FIXME: this should not be here, but called
-                # inside the task, or the dag, depending on the level
-                if dag._on_task_failure:
-                    dag._on_task_failure(t)
             else:
+                new_status = TaskStatus.Executed
+
                 try:
                     t.exec_status = new_status
-                except Exception as e:
+                except Exception:
                     tr = traceback.format_exc()
                     exceptions.append(traceback_str=tr, task_str=repr(t))
 
-                if dag._on_task_finish:
-                    dag._on_task_finish(t)
-            finally:
-                now = datetime.now()
-                did_execute = new_status == TaskStatus.Executed
-                elapsed = (now - then).total_seconds()
-                report = {'name': t.name,
-                          'Ran?': did_execute,
-                          'Elapsed (s)': elapsed if did_execute else 0}
-                task_reports.append(Row(report))
+                task_reports.append(report)
 
         if exceptions:
             raise DAGBuildError('DAG build failed, the following '
@@ -94,22 +82,16 @@ class Serial(Executor):
                                 'execution):\n{}'
                                 .format(str(exceptions)))
 
-        build_report = BuildReport(task_reports)
-
-        self._logger.info(' DAG report:\n{}'.format(repr(build_report)))
-
-        # TODO: this should be moved to the superclass, should be like
-        # a cleanup function, add a test to verify that this happens
-        # even if execution fails
-        for client in dag.clients.values():
-            client.close()
-
         if self.logging_directory:
             logger_handler.remove()
 
-        return build_report
+        # only close when tasks are executed in this process (otherwise
+        # this won't have any effect anyway)
+        if not self._execute_callables_in_subprocess:
+            for client in self.clients.values():
+                client.close()
 
-    # __getstate__ and __setstate__ are needed to make this picklable
+        return task_reports
 
     def __getstate__(self):
         state = self.__dict__.copy()
