@@ -75,10 +75,6 @@ class MetaProduct:
         # TODO: validate same task
         return self.products[0].task
 
-    @property
-    def name(self):
-        return ', '.join([p.name for p in self.products])
-
     @task.setter
     def task(self, value):
         for p in self.products:
@@ -101,9 +97,9 @@ class MetaProduct:
         for product in self.products:
             product.delete(force)
 
-    def _outdated(self):
-        return (self._outdated_data_dependencies()
-                or self._outdated_code_dependency())
+    def _is_outdated(self, outdated_by_code=True):
+        return any([p._is_outdated(outdated_by_code=outdated_by_code)
+                    for p in self.products])
 
     def _outdated_data_dependencies(self):
         return any([p._outdated_data_dependencies()
