@@ -23,6 +23,26 @@ Optional:
 NOTE: Params trigger different data output (and should make tasks outdated),
 Tasks constructor args (such as chunksize in SQLDump) should not change
 the output, hence shoulf not make tasks outdated
+
+Task Status lifecycle:
+
+* They all start on WaitingRender (set in __init__)
+* (dag.render() is called)
+* Tasks move to:
+    * If sucessful render: TaskStatus.Skipped, WaitingExecution or
+        WaitingUpstream
+    * If failed render: ErroredRender or AbortedRender
+    * This happens in Task.render()
+    * AbortedRender is not set directly, but by propagating downstream
+* (dag.build() is called)
+* Check that all TaskStatus are either WaitingExecution, WaitingUpstream
+    or Skipped (currently done in Executor)
+* Once dag.build() starts, the executor must call task.build() inside a
+    try-catch statement to set status to Executed or Errored
+* When tasks are set to Errored, Abort is set to downstream tasks
+
+TODO: describe BrokenProcesssPool
+
 """
 import traceback
 import abc
@@ -30,13 +50,13 @@ import logging
 import warnings
 from datetime import datetime
 from ploomber.products import Product, MetaProduct
-from ploomber.dag import DAG
+from ploomber.dag.DAG import DAG
 from ploomber.exceptions import TaskBuildError
 from ploomber.tasks.TaskGroup import TaskGroup
 from ploomber.constants import TaskStatus
 from ploomber.tasks.Upstream import Upstream
 from ploomber.tasks.Params import Params
-from ploomber.Table import Row
+from ploomber.Table import TaskReport, Row
 from ploomber.sources.sources import Source
 from ploomber.util import isiterable
 from ploomber.util.util import callback_check
@@ -102,11 +122,12 @@ class Task(abc.ABC):
             elif hasattr(source, '__name__'):
                 self._name = source.__name__
             else:
-                raise AttributeError('name can ony be None if the souce '
-                                     'has a "name" attribute such as a '
-                                     'Placeholder (returned from SourceLoader)'
-                                     ' or pathlib.Path objects, or a '
-                                     '"__name__" attribute (Python functions)')
+                raise AttributeError('Task name can only be None if the '
+                                     'source object has a "name" attribute: '
+                                     'Placeholder objects (returned from '
+                                     'SourceLoader) or pathlib.Path objects. '
+                                     'Python functions also work since they '
+                                     'have a "__name__" attribute')
         else:
             self._name = name
 
@@ -158,22 +179,32 @@ class Task(abc.ABC):
         self._on_finish = None
         self._on_failure = None
         self._on_render = None
-        self._available_callback_kwargs = {'task': self,
-                                           'client': self.client,
-                                           'product': self.product}
+
+    @property
+    def _available_callback_kwargs(self):
+        # make it a property so we always get the latest value for self.client
+        # given that could be None during init
+        return {'task': self,
+                'client': self.client,
+                'product': self.product}
 
     @property
     def name(self):
-        """A str that represents the name of the task
+        """
+        A str that represents the name of the task, you can access tasks
+        in a dag using dag['some_name']
         """
         return self._name
 
     @property
     def source(self):
         """
-        A code object which represents what will be run upn task execution,
-        for tasks that do not take source code as parameter (such as
-        PostgresCopyFrom), the source object will be a different thing
+        Source is used by the task to compute its output, for most cases
+        this is source code, for example PythonCallable takes a function
+        as source and SQLScript takes a string with SQL code as source.
+        But other tasks might take non-code objects as source, for example,
+        PostgresCopyFrom takes a path to a file. If source represents code
+        doing str(task.source) will return the string representation
         """
         return self._source
 
@@ -183,18 +214,10 @@ class Task(abc.ABC):
         """
         return self._product
 
-    # FIXME: remove, only keep source
-    @property
-    def source_code(self):
-        """
-        A str with the source for that this task will run on execution, if
-        templated, it is only available after rendering
-        """
-        return str(self.source)
-
     @property
     def upstream(self):
-        """{task names} -> [task objects] mapping for upstream dependencies
+        """
+        A mapping for upstream dependencies {task name} -> [task obkect]
         """
         # this is jus syntactic sugar, upstream relations are tracked by the
         # DAG object
@@ -309,28 +332,52 @@ class Task(abc.ABC):
 
     @exec_status.setter
     def exec_status(self, value):
+        # FIXME: this should only be used for th eexecutor to report back
+        # status Executed or Errored, reject all other cases, those are handled
+        # internally
         if value not in list(TaskStatus):
             raise ValueError('Setting task.exec_status to an unknown '
                              'value: %s', value)
 
         self._logger.debug('Setting "%s" status to %s', self.name, value)
         self._exec_status = value
+
+        # process might crash, propagate now or changes might not be
+        # reflected (e.g. if a Task is marked as Aborted, all downtream
+        # tasks should be marked as aborted as well)
         self._update_downstream_status()
 
+        # FIXME: this is inefficient, it is better to traverse
+        # the dag in topological order but exclude nodes not affected by
+        # this change
+
         # TODO: move this to build method? do we really need to run this in
         # the main process?
         if value == TaskStatus.Executed:
             # run on finish first, if this fails, we don't want to save
             # metadata
+            # Exceptions are *not* silenced here
             self._run_on_finish()
 
-            self.product._save_metadata(self.source_code)
+            self.product._save_metadata(str(self.source))
+
+            # NOTE: for most Products, it's ok to do this check before
+            # saving metadata, but not for GenericProduct, since the way
+            # exists() works is by checking metadata, so we have to do it
+            # after saving metadata for it to work
+            if not self.product.exists():
+                raise TaskBuildError('Error building task "{}": '
+                                     'the task ran successfully but product '
+                                     '"{}" does not exist yet '
+                                     '(task.product.exists() returned False)'
+                                     .format(self, self.product))
 
         elif value == TaskStatus.Errored:
+            # Exceptions here are silenced
             self._run_on_failure()
 
     def build(self, force=False, within_dag=False):
-        """Run the task if needed by checking its dependencies
+        """build the task
 
         Returns
         -------
@@ -342,10 +389,8 @@ class Task(abc.ABC):
         else:
             try:
                 res = self._build(force)
-            except Exception as e:
+            finally:
                 self.exec_status = TaskStatus.Errored
-                raise type(e)('Error calling build on task {}'
-                              .format(self.name)) from e
 
             self.exec_status = TaskStatus.Executed
             self.product._clear_cached_status()
@@ -356,101 +401,61 @@ class Task(abc.ABC):
         # cannot keep running, we depend on the render step to get all the
         # parameters resolved (params, upstream, product)
         if self.exec_status == TaskStatus.WaitingRender:
-            raise TaskBuildError('Cannot build task that has not been '
-                                 'rendered, call DAG.render() first')
+            raise TaskBuildError('Error building task "{}". '
+                                 'Cannot build task that has not been '
+                                 'rendered, call DAG.render() first'
+                                 .format(self.name))
 
         elif self.exec_status == TaskStatus.Aborted:
-            raise TaskBuildError('Attempted to run task "{}", which has '
-                                 'status TaskStatus.Aborted'
+            raise TaskBuildError('Attempted to run task "{}", whose '
+                                 'status is TaskStatus.Aborted'
+                                 .format(self.name))
+        elif self.exec_status == TaskStatus.Skipped and not force:
+            raise TaskBuildError('Attempted to run task "{}", whose '
+                                 'status TaskStatus.Skipped. Use force=True '
+                                 'if you want to execute it anyway'
                                  .format(self.name))
 
         # NOTE: should i fetch metadata here? I need to make sure I have
         # the latest before building
 
         if force:
-            self._logger.info('Forcing run "%s", skipping checks...',
+            self._logger.info('Forcing run "%s", status ignored...',
                               self.name)
-            run = True
-        else:
-            run = self.should_execute()
 
-        if run:
-            self._logger.info('Starting execution: %s', repr(self))
+        self._logger.info('Starting execution: %s', repr(self))
 
-            then = datetime.now()
+        then = datetime.now()
+        self.run()
+        now = datetime.now()
 
-            self.run()
+        elapsed = (now - then).total_seconds()
+        self._logger.info('Done. Operation took {:.1f} seconds'
+                          .format(elapsed))
 
-            now = datetime.now()
-            elapsed = (now - then).total_seconds()
-            self._logger.info('Done. Operation took {:.1f} seconds'
-                              .format(elapsed))
+        # TODO: also check that the Products were updated:
+        # if they did not exist, they must exist now, if they alredy
+        # exist, timestamp must be recent equal to the datetime.now()
+        # used. maybe run fetch metadata again and validate?
 
-            # TODO: also check that the Products were updated:
-            # if they did not exist, they must exist now, if they alredy
-            # exist, timestamp must be recent equal to the datetime.now()
-            # used. maybe run fetch metadata again and validate?
+        return TaskReport.with_data(name=self.name, ran=True, elapsed=elapsed)
 
-            if not self.product.exists():
-                raise TaskBuildError('Error building task "{}": '
-                                     'the task ran successfully but product '
-                                     '"{}" does not exist yet '
-                                     '(task.product.exist() returned False)'
-                                     .format(self, self.product))
-
-        # NOTE: return the TaskReport here?
-        return TaskStatus.Executed if run else TaskStatus.Skipped
-
-    def should_execute(self):
-        """
-        Given current conditions, determine if calling Task.build() should
-        actually run the Task.
-
-        Returns
-        -------
-        bool
-            True if the Task should execute, False otherwise
-        """
-        run = False
-
-        self._logger.info('Checking status for task "%s"', self.name)
-
-        # check product...
-        p_exists = self.product.exists()
-
-        # check dependencies only if the product exists
-        if p_exists:
-
-            outdated_data_deps = self.product._outdated_data_dependencies()
-            outdated_code_dep = self.product._outdated_code_dependency()
-
-            if outdated_data_deps:
-                run = True
-                self._logger.info('Outdated data deps...')
-            else:
-                self._logger.info('Up-to-date data deps...')
-
-            if outdated_code_dep:
-                run = True
-                self._logger.info('Outdated code dep...')
-            else:
-                self._logger.info('Up-to-date code dep...')
-        else:
-            run = True
-
-            # just log why it will run
-            if not p_exists:
-                self._logger.info('Product does not exist...')
-
-        self._logger.info('Should run? %s', run)
-
-        return run
-
-    def render(self):
+    def render(self, force=False, outdated_by_code=True):
         """
         Renders code and product, all upstream tasks must have been rendered
         first, for that reason, this method will usually not be called
         directly but via DAG.render(), which renders in the right order
+
+        Parameters
+        ----------
+        force : bool
+            If True, mark status as WaitingExecution/WaitingUpstream even if
+            the task is up to, otherwise up to date tasks are marked as
+            Skipped
+
+        outdated_by_code : str
+            Factors to determine if Task.product is marked outdated when source
+            code changes. Otherwise just the upstream timestamps are used.
         """
         self._logger.debug('Calling render on task %s', self.name)
 
@@ -482,11 +487,42 @@ class Task(abc.ABC):
                           ' check the full traceback above for details'
                           .format(repr(self), self.params)) from e
 
-        self.exec_status = (TaskStatus.WaitingExecution
-                            if not self.upstream
-                            else TaskStatus.WaitingUpstream)
+        # Maybe set ._exec_status directly, since no downstream propagation
+        # is needed here.
+        is_outdated = (self.product
+                       ._is_outdated(outdated_by_code=outdated_by_code))
+
+        if not self.upstream:
+            if not is_outdated and not force:
+                self._exec_status = TaskStatus.Skipped
+            else:
+                self._exec_status = TaskStatus.WaitingExecution
+        else:
+            all_upstream_done = all([t.exec_status
+                                     in {TaskStatus.Executed,
+                                         TaskStatus.Skipped}
+                                     for t in self.upstream.values()])
+
+            if all_upstream_done and is_outdated:
+                self._exec_status = TaskStatus.WaitingExecution
+            elif all_upstream_done and not is_outdated and not force:
+                self._exec_status = TaskStatus.Skipped
+            else:
+                self._exec_status = TaskStatus.WaitingUpstream
+
+        self._validate_render()
         self._run_on_render()
 
+    def _validate_render(self):
+        """
+        This hook is executed after rendering, it can be used to perform
+        validation on parameters that might cause runtime errors. e.g.
+        if is used by PythonCallable to check if the function signature
+        matches parameters passed. Note: if is executed before the
+        user-supplied on_render hook
+        """
+        pass
+
     def set_upstream(self, other):
         self.dag._add_edge(other, self)
 
@@ -500,7 +536,7 @@ class Task(abc.ABC):
 
         Source code:
         {}
-        """.format(self.params, self.product, self.source_code)
+        """.format(self.params, self.product, str(self.source))
 
         print(plan)
 
@@ -512,6 +548,12 @@ class Task(abc.ABC):
         data = {}
 
         data['name'] = self.name
+        data['type'] = type(self).__name__
+        data['status'] = self.exec_status.name
+        # FIXME: all tasks should have a client property
+        data['client'] = (repr(self.client)
+                          if hasattr(self, 'client')
+                          else None)
 
         if p.metadata.timestamp is not None:
             dt = datetime.fromtimestamp(p.metadata.timestamp)
@@ -521,20 +563,40 @@ class Task(abc.ABC):
         else:
             data['Last updated'] = 'Has not been run'
 
-        data['Outdated dependencies'] = p._outdated_data_dependencies()
+        outd_data = p._outdated_data_dependencies()
         outd_code = p._outdated_code_dependency()
+
+        outd = False
+
+        if outd_code:
+            outd = 'Source code'
+
+        if outd_data:
+            if not outd:
+                outd = 'Upstream'
+            else:
+                outd += ' & Upstream'
+
+        data['Outdated?'] = outd
+
+        data['Outdated dependencies'] = outd_data
         data['Outdated code'] = outd_code
 
         if outd_code and return_code_diff:
             data['Code diff'] = (self.dag
                                  .differ
                                  .get_diff(p.metadata.stored_source_code,
-                                           self.source_code,
-                                           language=self.source.language))
+                                           str(self.source),
+                                           extension=self.source.extension))
         else:
             outd_code = ''
 
+        data['Product type'] = type(self.product).__name__
         data['Product'] = str(self.product)
+        # FIXME: all products should have a client property
+        data['Product client'] = (repr(self.product.client)
+                                  if hasattr(self.product, 'client')
+                                  else None)
         data['Doc (short)'] = self.source.doc_short
         data['Location'] = self.source.loc
 
@@ -546,7 +608,7 @@ class Task(abc.ABC):
         attributes
         """
         return dict(name=self.name, product=str(self.product),
-                    source_code=self.source_code)
+                    source_code=str(self.source))
 
     def _render_product(self):
         params_names = list(self.params)
@@ -581,6 +643,9 @@ class Task(abc.ABC):
         return downstream
 
     def _update_downstream_status(self):
+        # FIXME: this is inefficient, it is better to traverse
+        # the dag in topological order but exclude nodes not affected by
+        # this change
         # TODO: move to DAG
         def update_status(task):
             any_upstream_errored_or_aborted = any([t.exec_status
